
#include "ad++.h"

const int N = 8 ;

int main(int argc, char** argv) {
  AdlibInit(argc, argv) ;

  Procs1 p(4) ;
  ON(p) {
    CyclicRange x(N, p.dim(0)) ;
    Range z(N) ;

    // Initialization

    Array2<float> a(z, x) ;
    Array1<float> b(z) ;  // buffer

    Index m(x) ;
    OVERALL(m) {
      for(int i = 0 ; i < N ; i++)
        if (m >= i)
          a(z(i), m) = i + 1 ;
        else
          a(z(i), m) = m + 1 ;
    } ALLOVER(i) ;

    gprintf("initial a:\n") ;
    aprintf("%7.3f%*N", a, N) ;

    // Do decomposition

    for(int k = 0 ; k < N - 1 ; k++) {

      Location l(x, k) ;
      AT(l) {
        const float d = sqrt(a(z(k), l)) ;

        a(z(k), l) = d ;
        for(int s = k + 1 ; s < N ; s++)
          a(z(s), l) /= d ;
      } TA(l) ;

      remap(b.sect(z.triplet(k + 1, N - 1)),
            a.sect(z.triplet(k + 1, N - 1), x(k))) ;

      Index m(x.triplet(k + 1, N - 1)) ;
      OVERALL(m) {
        // NB: `x.idx(m)' is currently expressed as k + 1 + (int) m

        for(int i = k + 1 + m ; i < N ; i++)
          a(z(i), m) -= b(z(i)) * b(z(k + 1 + m)) ;
      } ALLOVER(m) ;

      gprintf("after iteration %d:\n", k) ;
      aprintf("%7.3f%*N", a, N) ;
    }

    Location l(x, N - 1) ;
    AT(l) {
      a(z(N - 1), l) = sqrt(a(z(N - 1), l)) ;
    } TA(l) ;

    gprintf("final a:\n") ;
    aprintf("%7.3f%*N", a, N) ;

  } NO(p) ;

  AdlibFinalize() ;
}

