/*
 * File         : spmd_Procs.C
 * Author       : Guansong Zhang
 * Created      : Mon Oct 13 12:06:49 1997
 * Revision     : $Revision: 1.1 $
 * Updated      : $Date: 1998/01/30 17:53:30 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1997
 */

#include "spmd_Procs.h"
#include "adlib.h"
#include "iostream.h"

extern jfieldID hCPPObjID;
#include "jctable.h"
extern jctable<Dimension> spmd_Dimension_Table;
extern jctable<Group> spmd_Group_Table;

Dimension * spmd_Dimension_ResolvePtr(JNIEnv *, jobject);

/*ResolvePtr
 */
Group * spmd_Group_ResolvePtr(JNIEnv *, jobject);

/*
 * Class:     spmd_Procs
 * Method:    constructor
 * Signature: ([I)V
 */
JNIEXPORT void JNICALL Java_spmd_Procs_constructor
  (JNIEnv *env, jobject jthis, jintArray grid)
{
  int rank=(env->GetArrayLength(grid));
  int * dimSize=new int [rank];

  jboolean isCopy;
  jint * jdimSize;

  jdimSize=env->GetIntArrayElements(grid, &isCopy);

  for (int i=0; i<rank; i++) 
    *(dimSize+i)=*(jdimSize+i);

  env->ReleaseIntArrayElements(grid, jdimSize, 0);

  Procs *procsPtr = new Procs(rank, dimSize);

  if (procsPtr==NULL) {
    jclass cError=env->FindClass("java/lang/OutOfMemeoryError");
    env->ThrowNew(cError,"adlib memory allocation failed");
    return;
  }

  Group *cppGroup=new Group(*procsPtr);

  int pos = env->GetIntField(jthis, hCPPObjID);
  delete (spmd_Group_Table.ObjPtr[pos]);

  spmd_Group_Table.ObjPtr[pos]=cppGroup;

  if (spmd_Group_Table.ObjPtr[pos]==NULL) {
    jclass cError=env->FindClass("java/lang/OutOfMemeoryError");
    env->ThrowNew(cError,"adlib memory allocation failed");
    return;
  }

  //spmd_Group_IncRefCntr(pos);

  delete [] dimSize; 

  env->SetIntField(jthis, hCPPObjID, pos);

}

/*
 * Class:     spmd_Procs
 * Method:    finalize
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_spmd_Procs_finalize
  (JNIEnv * env, jobject jthis)
{
  int pos = env->GetIntField(jthis, hCPPObjID);
  int rc=spmd_Group_Table.DecRefCntr(pos);
  if (rc==0) {
    Group * groupPtr=spmd_Group_Table.ObjPtr[pos];
    Procs * procsPtr=(Procs *)groupPtr->prc();
    delete procsPtr;
    delete groupPtr;
    groupPtr=NULL;
  }
};

/*
 * Class:     spmd_Procs
 * Method:    dimension
 * Signature: (I)Lspmd/Dimension;
 */
JNIEXPORT jobject JNICALL Java_spmd_Procs_dimension
  (JNIEnv * env, jobject jthis, jint d)
{

  //Dimension *newCPPdimPtr=new Dimension();

  Dimension CPPdim = (spmd_Group_ResolvePtr(env, jthis)->prc())->dim(d);
  
  //long pos=spmd_Dimension_NewHandle(newCPPdimPtr);

  //printf("dim pos %ld, address %x\n", pos, spmd_Dimension_ObjPtr[pos]);
  //printf("is this dim? crd %d\n", spmd_Dimension_ObjPtr[pos]->crd());

  jclass ccDimension=NULL;
  ccDimension=env->FindClass("spmd/Dimension");
  if (env->ExceptionOccurred()) return 0;
  jmethodID handleConstructorID = env->GetMethodID(ccDimension, "<init>", "()V");
  jobject jDimension=env->NewObject(ccDimension, handleConstructorID);

  Dimension * newCPPdimPtr=spmd_Dimension_ResolvePtr(env, jDimension);

  *newCPPdimPtr=CPPdim;

  return(jDimension);
};

