
// Operations on cyclic distributed ranges.


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
//                                                                     //
// class `CyclicRangeRep': Cyclic-distributed template range.          //
//                                                                     //
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

class CyclicRangeRep : public RangeRep {
public :
  CyclicRangeRep(RangeRep* _kernel, const int _numLayers) :
          RangeRep(_numLayers * _kernel->size(),
                   _kernel, _kernel->lev() + 1, _kernel->dim()),
          numLayers(_numLayers), numProcs(_kernel->size()) {
    kernel->incr() ;
  }

  ~CyclicRangeRep() {
    kernel->decr() ;
  }

  RangeRep* subrng(const int _extent, const int _base, const int _stride) ;

  Format format() const ;

  int ker_glb(const int glb, int* sub) const ;

  int block(const int ker_glb, int* sub, int* count) const ;
  int block_step(int* sub_stp) const ;

  int width() const ;
  StrideRep* vector(StrideRep* _kernel, const int _length) const ;

  int glb(const int sub, const int ker_glb) const ;

  void show() ;

protected :
  int numLayers, numProcs ;

MEMORY_POOL_DECS
} ;


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
//                                                                     //
// class `CyclicSubRangeRep': cyclic-distributed subrange.             //
//                                                                     //
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

class CyclicSubRangeRep : public RangeRep {
public :
  CyclicSubRangeRep(const int _extent,
		   const int _base, const int _stride,
		   CyclicRangeRep* _parent,
		   RangeRep* _kernel, const int _numLayers) :
          RangeRep(_extent, _kernel, _kernel->lev() + 1, _kernel->dim()),
          numLayers(_numLayers), numProcs(_kernel->size()),
	  base(_base), stride(_stride), parent(_parent) {
    parent->incr() ;
    euler() ;
  }

  ~CyclicSubRangeRep() {
    parent->decr() ;
  }

  RangeRep* subrng(const int _extent, const int _base, const int _stride) ;

  RangeRep* tem() ;
  int bas() const ;
  int str() const ;

  Format format() const ;

  int ker_glb(const int glb, int* sub) const ;

  int block(const int ker_glb, int* sub, int* count) const ;
  int block_step(int* sub_stp) const ;

  RangeRep* subker(int* ker_bas, int* ker_str) const ;

  int width() const ;
  StrideRep* vector(StrideRep* _kernel, const int _length) const ;

  int glb(const int sub, const int ker_glb) const ;

  void show() ;

protected :
  int numLayers, numProcs ;
  int base, stride ;
  CyclicRangeRep* parent ;
  int f, f_s, k_step ;

private :
  void euler() ;

MEMORY_POOL_DECS
} ;


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
//                                                                     //
// classes `ADAPTCyclicRangeRep', `ADAPTCyclicSubRangeRep':            //
//     Cyclic-distributed ranges yielding arrays with ADAPT layout.    //
//                                                                     //
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

class ADAPTCyclicRangeRep : public CyclicRangeRep {
public :
  ADAPTCyclicRangeRep(RangeRep* _kernel, const int _numLayers) :
          CyclicRangeRep(_kernel, _numLayers) {}

  RangeRep* subrng(const int _extent, const int _base, const int _stride) ;
} ;

class ADAPTCyclicSubRangeRep : public CyclicSubRangeRep {
public :
  ADAPTCyclicSubRangeRep(const int _extent,
                         const int _base, const int _stride,
                         CyclicRangeRep* _parent,
                         RangeRep* _kernel, const int _numLayers) :
          CyclicSubRangeRep(_extent, _base, _stride,
                            _parent, _kernel, _numLayers) {}

  RangeRep* subrng(const int _extent, const int _base, const int _stride) ;

  int width() const ;
} ;


// -------------------------------------------------------------
// Things to do
//
//   Revert to `CyclicSubRangeRep' derived from `CyclicRangeRep'.

