
// Operations on stepped block distributed ranges.

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
//                                                                     //
// class `StepRangeRep': stepped block-distributed template range.     //
//                                                                     //
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

class StepRangeRep : public RangeRep {
public :
  StepRangeRep(RangeRep* _kernel, const int _blockSize, const int _stepCrd) :
          RangeRep(_blockSize * _kernel->size() + _stepCrd,
                   _kernel, _kernel->lev() + 1, _kernel->dim()),
          blockSize(_blockSize), stepCrd(_stepCrd),
          stepSub(_stepCrd * (_blockSize + 1)) {
    kernel->incr() ;
  }

  ~StepRangeRep() {
    kernel->decr() ;
  }

  RangeRep* subrng(const int _extent, const int _base, const int _stride) ;

  Format format() const ;

  int ker_glb(const int glb, int* sub) const ;

  int block(const int ker_glb, int* sub, int* count) const ;
  int block_step(int* sub_stp) const ;

  int width() const ;
  StrideRep* vector(StrideRep* _kernel, const int _len) const ;

  int glb(const int sub, const int ker_glb) const ;

  void show() ;

protected :

  int to_ker_glb(const int sub) const {
    if(sub < stepSub)
      return sub / (blockSize + 1) ;
    else
      return (sub - stepCrd) / blockSize ;
  }

  int blockSize, stepCrd, stepSub ;

MEMORY_POOL_DECS
} ;


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
//                                                                     //
// class `StepSubRangeRep': block-distributed subrange.                //
//                                                                     //
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

class StepSubRangeRep : public RangeRep {
public :
  StepSubRangeRep(const int _extent,
		  const int _base, const int _stride,
		  StepRangeRep* _parent,
		  RangeRep* _kernel,
                  const int _blockSize, const int _stepCrd) :
          RangeRep(_extent, _kernel, _kernel->lev() + 1, _kernel->dim()),
          blockSize(_blockSize), stepCrd(_stepCrd),
          stepSub(_stepCrd * (_blockSize + 1)),
	  base(_base), stride(_stride), parent(_parent) {
    parent->incr() ;

    if(stride > 0) {
      ker_lo = to_ker_glb(base) ;
      ker_hi  = to_ker_glb(base + stride * (extent - 1)) ;
    }
    else {
      ker_hi = to_ker_glb(base) ;
      ker_lo  = to_ker_glb(base + stride * (extent - 1)) ;
    }
  }

  ~StepSubRangeRep() {
    parent->decr() ;
  }

  RangeRep* subrng(const int _extent, const int _base, const int _stride) ;

  RangeRep* tem() ;
  int bas() const ;
  int str() const ;

  Format format() const ;

  int ker_glb(const int glb, int* sub) const ;

  int block(const int ker_glb, int* sub, int* count) const ;
  int block_step(int* sub_stp) const ;

  RangeRep* subker(int *ker_bas, int* ker_str) const ;

  int width() const ;
  StrideRep* vector(StrideRep* _kernel, const int _len) const ;

  int glb(const int sub, const int ker_glb) const ;

  void show() ;

protected :

  int to_ker_glb(const int sub) const {
    if(sub < stepSub)
      return sub / (blockSize + 1) ;
    else
      return (sub - stepCrd) / blockSize ;
  }

  int blockSize, stepCrd, stepSub ;
  int base, stride ;
  StepRangeRep* parent ;

  int ker_lo, ker_hi ;

MEMORY_POOL_DECS
} ;


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
//                                                                     //
// classes `PackedStepRangeRep', `PackedStepSubRangeRep':              //
//      Step-distributed ranges yielding arrays with ``packed''        //
//      layout.                                                        //
//                                                                     //
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

class PackedStepRangeRep : public StepRangeRep {
public :
  PackedStepRangeRep(RangeRep* _kernel,
                     const int _blockSize, const int _stepCrd) :
          StepRangeRep(_kernel, _blockSize, _stepCrd) {}

  RangeRep* subrng(const int _extent, const int _base, const int _stride) ;
} ;


class PackedStepSubRangeRep : public StepSubRangeRep {
public :
  PackedStepSubRangeRep(const int _extent,
                        const int _base, const int _stride,
                        StepRangeRep* _parent,
                        RangeRep* _kernel,
                        const int _blockSize, const int _stepCrd) :
        StepSubRangeRep(_extent, _base, _stride, _parent,
                        _kernel, _blockSize, _stepCrd),
        packing(_stride > 0 ? _stride : -_stride) {}

  RangeRep* subrng(const int _extent, const int _base, const int _stride) ;

  int width() const ;
  StrideRep* vector(StrideRep* _kernel, const int _len) const ;

protected :
  int packing ;

MEMORY_POOL_DECS
} ;


// -------------------------------------------------------------
// Things to do
//
//   Revert to `StepSubRangeRep' derived from `StepRangeRep'.

