
// Operations on collapsed ranges.

#include <stdlib.h>
#include <stdio.h>
#include <memory.h>
#include "globals.h"
#include "Procs.h"
#include "Range.h"


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
//                                                                     //
// class `CollapsedRangeRep': collapsed template range.                //
//                                                                     //
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

RangeRep* CollapsedRangeRep :: subrng(const int _extent,
                                      const int _base, const int _stride) {
  return new CollapsedSubRangeRep(_extent, _base, _stride, this, kernel) ;
}

Format CollapsedRangeRep :: format() const {
  return DIST_COLLAPSED ;
}

int CollapsedRangeRep :: ker_glb(const int glb, int* sub) const {
  *sub = glb ;

  return 0 ;
}

int CollapsedRangeRep :: block(const int, int* sub, int* count) const {
  *sub   = 0 ;
  *count = extent ;

  return 0 ;
}

int CollapsedRangeRep :: block_step(int* sub_stp) const {
  *sub_stp = 1 ;

  return 1 ;
}

int CollapsedRangeRep :: width() const {return extent ; }

StrideRep* CollapsedRangeRep :: vector(StrideRep* _kernel,
                                       const int _length) const {
  return new StrideRep(_kernel, _length) ;
}

void CollapsedRangeRep :: show() {
  AdlibPrintf("CollapsedRangeRep :: show: size = %d ker = %p lev = %d dim = %p\n",
              extent, kernel, level, (DimensionRep*) dimension) ; 
}

MEMORY_POOL_DEFN(CollapsedRangeRep)


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
//                                                                     //
// class `CollapsedSubRangeRep': subrange of a collapsed range.        //
//                                                                     //
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

RangeRep* CollapsedSubRangeRep :: subrng(const int _extent,
                                         const int _base, const int _stride) {
  return new CollapsedSubRangeRep(_extent,
                                  base + stride * _base, stride * _stride,
                                  parent, kernel) ;
}

RangeRep* CollapsedSubRangeRep :: tem() {return parent ; }

int CollapsedSubRangeRep :: bas() const {return base ; }

int CollapsedSubRangeRep :: str() const {return stride ; }

Format CollapsedSubRangeRep :: format() const {
  return DIST_COLLAPSED ;
}

int CollapsedSubRangeRep :: ker_glb(const int glb, int* sub) const {
  *sub = base + stride * glb ;

  return 0 ;
}

int CollapsedSubRangeRep :: block(const int,
                                  int* sub, int* count) const {
  *sub   = base ;
  *count = extent ;

  return 0 ;
}

int CollapsedSubRangeRep :: block_step(int* sub_stp) const {
  *sub_stp = stride ;

  return 1 ;
}

int CollapsedSubRangeRep :: width() const {return parent->size() ; }

StrideRep* CollapsedSubRangeRep :: vector(StrideRep* _kernel,
                                          const int _length) const {
  return new StrideRep(_kernel, _length) ;
}

int CollapsedSubRangeRep :: glb(const int sub, const int) const {
  return (sub - base) / stride ;
}

void CollapsedSubRangeRep :: show() {
  AdlibPrintf("CollapsedSubRangeRep :: show: size = %d ker = %p lev = %d dim = %p\n",
              extent, kernel, level, (DimensionRep*) dimension) ; 
}

MEMORY_POOL_DEFN(CollapsedSubRangeRep)


// -------------------------------------------------------------
// Things to do
//

