
// Operations on level-0 ranges.

#include <stdlib.h>
#include <stdio.h>
#include <memory.h>
#include "globals.h"
#include "Procs.h"
#include "Range.h"


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
//                                                                     //
// class `DimRangeRep': process dimension range.                       //
//                                                                     //
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

RangeRep* DimRangeRep :: subrng(const int _extent,
                                const int _base, const int _stride) {
  return new DimSubRangeRep(_extent, _base, _stride, this, dimension) ;
}

Format DimRangeRep :: format() const {
  return DIST_PRIMITIVE ;
}

int DimRangeRep :: crd(const int glb) const {return glb ; }

int DimRangeRep :: local(int* glb) const {
  *glb = dimension.crd() ;
  return 1 ;
}

void DimRangeRep :: show() {
  //AdlibPrintf("DimRangeRep :: show: size = %d ker = %p lev = %d dim = %p\n",
  //            extent, kernel, level, (DimensionRep*) dimension) ; 
  AdlibPrintf("DimRangeRep :: show: size = %d dim = %p\n",
              extent, (DimensionRep*) dimension) ; 
}

MEMORY_POOL_DEFN(DimRangeRep)


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
//                                                                     //
// class `DimSubRangeRep': process dimension subrange.                 //
//                                                                     //
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

RangeRep* DimSubRangeRep :: subrng(const int _extent,
                                   const int _base, const int _stride) {
  return new DimSubRangeRep(_extent, base + stride * _base, stride * _stride,
                            parent, dimension) ;
}

RangeRep* DimSubRangeRep :: tem() {return parent ; }

int DimSubRangeRep :: bas() const {return base ; }

int DimSubRangeRep :: str() const {return stride ; }

Format DimSubRangeRep :: format() const {
  return DIST_PRIMITIVE ;
}

int DimSubRangeRep :: crd(const int glb) const {
  return base + stride * glb ;
}

int DimSubRangeRep :: local(int* glb) const {
  const int offset = dimension.crd() - base ;
  if(offset % stride == 0) {
    *glb = offset / stride ;
    return *glb >= 0 && *glb < extent ;
  }
  else
    return 0 ;
}

int DimSubRangeRep :: glb(const int sub, const int) const {
  return (sub - base) / stride ;
}

void DimSubRangeRep :: show() {
  AdlibPrintf("DimSubRangeRep :: show: size = %d ker = %p lev = %d dim = %p\n",
              extent, kernel, level, (DimensionRep*) dimension) ; 
}

MEMORY_POOL_DEFN(DimSubRangeRep)


// -------------------------------------------------------------
// Things to do
//
//     Currently support collapsed dimensions through `dim == 0',
//         but don't support subranges of collapsed dimensions.

