/*
 * File         : Fox.java
 * Author       : Guansong Zhang
 * Created      : Wed Nov  5 17:10:16 1997
 * Revision     : $Revision: 1.5 $
 * Updated      : $Date: 1998/03/24 16:13:46 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1997
 */

import spmd.*;

class Fox {

  final static int P=1;
  final static int B=256;

  static Group root;

  public static void input1(Group p, Array4Float a) {
    // should be remaped from root
    for (int m=0; m<P; m++) {
      Location lm=a.range[0].location(m);     
      if (lm.at())
	for (int n=0; n<P; n++) {
	  Location ln=a.range[1].location(n);
	  if(ln.at())
	    for (int i=0; i<B; i++) {
	      Location li=a.range[2].location(i);
	      if(li.at())
		for (int j=0; j<B; j++) {
		  Location lj=a.range[3].location(j);
		  if(lj.at()) {
		    a.data[a.element(lm, ln, li, lj)]=2*(B*P-1)-(B*m+B*n+i+j);
		  } lj.ta();
		} li.ta();	  
	    } ln.ta();
	} lm.ta();
    }
  };

  public static void input0(Group p, Array4Float a) {
    // should be remaped from root
    for (int m=0; m<P; m++) {
      Location lm=a.range[0].location(m);     
      if (lm.at())
	for (int n=0; n<P; n++) {
	  Location ln=a.range[1].location(n);
	  if(ln.at())
	    for (int i=0; i<B; i++) {
	      Location li=a.range[2].location(i);
	      if(li.at())
		for (int j=0; j<B; j++) {
		  Location lj=a.range[3].location(j);
		  if(lj.at()) {
		    a.data[a.element(lm, ln, li, lj)]=B*m+B*n+i+j;
		  } lj.ta();
		} li.ta();	  
	    } ln.ta();
	} lm.ta();
    }
  };

  public static void output(Group p, String name, Array4Float a) {
    // p has one single process as root

    Range clp = new Range(P);
    Range z= new Range(B);
    Array4Float res = new Array4Float (clp, clp, z, z, p);

    Adlib.remap(res, a);

    if (p.on()) {
      System.out.println(name);

      for (int m=0; m<P; m++) {
	Location lm=res.range[0].location(m);     
	for (int i=0; i<B; i++) {
	  Location li=res.range[2].location(i);
	  for (int n=0; n<P; n++) {
	    Location ln=res.range[1].location(n);
	    for (int j=0; j<B; j++) {
	      Location lj=res.range[3].location(j);
	      System.out.print((int)res.data[res.element(lm, ln, li, lj)]+"  ");
	    }
	  }
	  System.out.println("|");
	}
      }
    } p.no();    
  };

  public static void matmul(Array2Float c, Array2Float a, Array2Float b) {

    /*
    for (int i=0; i<B; i++)
      for (int j=0; j<B; j++)
	for (int k=0; k<B; k++)
	  c.data[c.element(c.range[0].location(i), c.range[1].location(j))]+=
	    a.data[a.element(a.range[0].location(i), a.range[1].location(k))]*
	    b.data[b.element(b.range[0].location(k), b.range[1].location(j))];



    */
    Range c_r0=c.range[0];
    Range c_r1=c.range[1];
    Stride c_u0=c.stride[0];
    Stride c_u1=c.stride[1];

    Range a_r0=a.range[0];
    Range a_r1=a.range[1];
    Stride a_u0=a.stride[0];
    Stride a_u1=a.stride[1];

    Range b_r0=b.range[0];
    Range b_r1=b.range[1];
    Stride b_u0=b.stride[0];
    Stride b_u1=b.stride[1];

    final int i_c_bas=c_u0.disp(c_r0.bas());
    final int i_a_bas=a_u0.disp(a_r0.bas());
    final int i_c_stp=c_u0.disp_step(c_r0.str());
    final int i_a_stp=a_u0.disp_step(a_r0.str());

    final int j_c_bas=c_u1.disp(c_r1.bas());
    final int j_b_bas=b_u1.disp(b_r1.bas());
    final int j_c_stp=c_u1.disp_step(c_r1.str());
    final int j_b_stp=b_u1.disp_step(b_r1.str());

    final int k_a_bas=a_u1.disp(a_r1.bas());
    final int k_b_bas=b_u0.disp(b_r0.bas());
    final int k_a_stp=a_u1.disp_step(a_r1.str());
    final int k_b_stp=b_u0.disp_step(b_r0.str());


    int i_c_pos=i_c_bas;
    int i_a_pos=i_a_bas;

    for (int i=0; i<B; i++) {

      int j_c_pos=j_c_bas;
      int j_b_pos=j_b_bas;

      for (int j=0; j<B; j++) {

	int k_a_pos=k_a_bas;
	int k_b_pos=k_b_bas;

	for (int k=0; k<B; k++) {
	  c.data[i_c_pos+j_c_pos]+=
	    a.data[i_a_pos+k_a_pos]*
	    b.data[k_b_pos+j_b_pos];

	  k_a_pos+=k_a_stp;
	  k_b_pos+=k_b_stp;

	}

	j_c_pos+=j_c_stp;
	j_b_pos+=j_b_stp;

      }

      i_c_pos+=i_c_stp;
      i_a_pos+=i_a_stp;

    }

//     for (int i=0; i<B; i++) {
//       for (int j=0; j<B; j++) {
// 	for (int k=0; k<B; k++) {
// 	  c.data[c_u0.offset(c_r0.location(i))+c_u1.offset(c_r1.location(j))]+=
// 	    a.data[a_u0.offset(a_r0.location(i))+a_u1.offset(a_r1.location(k))]*
// 	    b.data[b_u0.offset(b_r0.location(k))+b_u1.offset(b_r1.location(j))];
// 	}
//       }
//     }



  }

  public static void main(String argv[]) {
    Adlib.Initialize(argv);

    int[] grid = {P, P};
    Procs p=new Procs(grid);

    Range x=p.dim(0);
    Range y=p.dim(1);

    root=p.divide(x.location(0)).divide(y.location(0));
      
    Range z=new Range(B);

    if(p.on()) {
      Array4Float a = new Array4Float(x,y,z,z,p);
      input0(root, a);
      //      output(root, "A=", a);
                  
      Array4Float b = new Array4Float(x,y,z,z,p);
      input1(root, b);
      //      output(root,"B=", b);

      Array4Float tmp = new Array4Float(x,y,z,z,p);

      Array4Float c = new Array4Float(x,y,z,z,p);

      Spmd.clock(0);

      for (int k=0; k<P; k++) {
	x.over(); {
	  int m=x.crd(); // coordinate in the range;
	  Location i=x.location(m);
	  Array2Float sub = new Array2Float(z,z,p.divide(i));
	    
	  //Broadcast submatrix in 'a' ...
	  Adlib.remap(sub, a.section(i, y.location((m+k)%P), z, z));

	  y.over(); {
	    int n=y.crd();
	    Location j=y.location(n);

	    //Local matrix multiprlication ...
	    matmul(c.section(i, j, z, z), sub, b.section(i, j, z, z));
	  } y.revo();
	} x.revo();
	  
	//Cyclic shift 'b' in 'y' dimension ...
	Adlib.shift(tmp, b, 1, 0, 0); // dst, src, shift, dim, mode;
	Adlib.copy(b, tmp);
      }

      //      output(root, "C=", c);

      Spmd.clock(1);

      
    } p.no();

    Adlib.Finalize();
    System.exit(0);
  }
}
