/*
 * File         : Jacobi.java
 * Author       : Guansong Zhang
 * Created      : Fri Jan 23 11:23:30 1998
 * Revision     : $Revision: 1.4 $
 * Updated      : $Date: 1998/02/20 17:01:47 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

import spmd.*;
import java.util.*;

class Jacobi {

  static final int n=1024;

  static Stack stk = new Stack();

  public static void main(String argv[]) {
    Adlib.Initialize(argv);

    Procs2 p=new Procs2(2,2);

    if (p.on()) {

      int [] w = {1,1};
      
      // enum Layout {LAY_STANDARD, LAY_PACKED, LAY_ADAPT, LAY_EXTENDED}

      BlockRange x = new BlockRange (n, p.dim(0), 3, 1, 1);
      BlockRange y = new BlockRange (n, p.dim(1), 3, 1, 1);

      Array2Float u = new Array2Float(x,y);

      Index i = new Index (x);
      Index j = new Index (y);

      i.dim().over();
      for (i.begin(); i.test(); i.next()) {
	j.dim().over();
	for (j.begin(); j.test(); j.next()) {
	  if (i.idx()==0 || i.idx()==n-1 || j.idx()==0 || j.idx()==n-1)
	    u.data[u.element(i, j)]=1;
	  else
	    u.data[u.element(i, j)]=0;
	} j.dim().revo();
      } i.dim().revo();

      Array2Float uold = new Array2Float(x, y);

      Index ic = new Index (x.subrng(n-2, 1));
      Index jc = new Index (y.subrng(n-2, 1));


      for (int iter=0; iter<100; iter++) {

	if (iter==10) 
	  Spmd.clock(0);

      	Adlib.copy(uold, u);
	Adlib.writeHalo(uold,w,w);

	/*
	ic.dim().over();
	for (ic.begin(); ic.test(); ic.next()) {
	  jc.dim().over();
	  for (jc.begin(); jc.test(); jc.next()) {
	    u.data[u.element(ic,jc)]
	      =(float)0.25*(uold.data[uold.element(ic.shift(-1),jc)]+
			    uold.data[uold.element(ic.shift(1),jc)]+
			    uold.data[uold.element(ic,jc.shift(-1))]+
			    uold.data[uold.element(ic,jc.shift(1))]);
	  } jc.dim().revo();
	} ic.dim().revo();
	*/

	float[] u_ptr=u.data;
	float[] uold_ptr=uold.data;

	jc.dim().over();
	
	for (jc.beginLocBlk(); jc.test(); jc.nextLocBlk()) {

	  int u1_bas=u.stride[1].offset(jc);
	  final int u1_stp=u.stride[1].step(jc);
	  final int u1_len=u.stride[1].len();

	  int uold1_bas=uold.stride[1].offset(jc);
	  final int uold1_stp=uold.stride[1].step(jc);
	  final int uold1_len=uold.stride[1].len();

	  final int jc_count=jc.count();

	  for (int jc_blk=0; jc_blk<jc_count; jc_blk++) {


	    //stk.push(Spmd.apg);

	    /* 
	    //Spmd.apg=(Group)Spmd.apg.clone();
	    //Spmd.apg.restrict(ic);
	    
	    //Spmd.apg=Spmd.apg.divide(ic);
	    */

	    //	    ic.dim().over();
	    for (ic.beginLocBlk(); ic.test(); ic.nextLocBlk()) {

	      u.stride[0].updateLocBlk(ic);
	      int u0_bas=u.stride[0].offset;
	      final int u0_stp=u.stride[0].step;
	      final int u0_len=u.stride[0].len;

	      uold.stride[0].updateLocBlk(ic);
	      int uold0_bas=uold.stride[0].offset;
	      final int uold0_stp=uold.stride[0].step;
	      final int uold0_len=uold.stride[0].len;

	      final int ic_count=ic.count();

	      for (int ic_blk=0; ic_blk<ic_count; ic_blk++) {
		int u_bas=u0_bas+u1_bas;
		int uold_bas=uold0_bas+uold1_bas;

 		u_ptr[u_bas]
		  =(uold_ptr[uold_bas-uold0_len]+
		    uold_ptr[uold_bas+uold0_len]+
 		    uold_ptr[uold_bas-uold0_len]+
 		    uold_ptr[uold_bas+uold0_len])*(float)0.25;

		u0_bas+=u0_stp;
		uold0_bas+=uold0_stp;
	      }


	    } //ic.dim().revo();

	    //Spmd.apg=(Group)stk.pop();

	    u1_bas+=u1_stp;
	    uold1_bas+=uold1_stp;
	  }

	} jc.dim().revo();

      }

      Spmd.clock(1);

      //Spmd.rout.println("iteration = "+iter);
      //Adlib.aprintf("%7.3f%"+n+"N", u);

    } p.no();

    Adlib.Finalize();
  }
}
